@@ -33,7 +33,7 @@ def activate_line(x, y, z, command, tile_type=""):
 		return_num = int(command)
 
 	elif switch == "addpix": #add a picture to the tile.
-		g.maps[z].field[y][x].add_pix(g.tiles[command.split(None, 2)[1]])
+		g.maps[z].field[y][x].add_pix(g.tiles[command.split(None, 1)[1]])
 		return_num = 1
 
 	elif switch == "addskill": #add a skill
@@ -51,8 +51,8 @@ def activate_line(x, y, z, command, tile_type=""):
 			temp = monster.find_level_monster(i)
 		else:
 			#battle named monster
-			for monster_num in range(len(monster.monsters)):
-				if monster.monsters[monster_num].name.lower() == \
+			for monster_num in range(len(monster.monster_groups)):
+				if monster.monster_groups[monster_num].name.lower() == \
 				command.split(None, 1)[1].lower():
 					temp = monster_num
 					break
@@ -65,7 +65,7 @@ def activate_line(x, y, z, command, tile_type=""):
 	elif switch == "delpix": #remove a picture from the tile.
 		try:
 			g.maps[z].field[y][x].del_pix(
-				g.tiles[command.split(None, 2)[1]])
+				g.tiles[command.split(None, 1)[1]])
 			main.refreshmap()
 			return_num = 1
 		except ValueError:
@@ -141,12 +141,8 @@ def activate_line(x, y, z, command, tile_type=""):
 			g.gold = int(g.gold) + int(set_to)
 			if 1 > g.gold: g.gold = 0
 		elif switch2 == "exp":
-			g.exp = g.exp + int(set_to)
+			g.add_exp(int(set_to))
 			if 1 > g.exp: g.exp = 0
-			if int(g.exp) >= int(10*(g.level + 1)*(g.level + 1)):
-				g.level=g.level+1
-				for i in g.levelup:
-					activate_line(x, y, z, i)
 		elif switch2 == "skillpoints":
 			g.skillpoints = g.skillpoints + int(set_to)
 			if 1 > g.skillpoints: g.skillpoints = 0
@@ -269,6 +265,9 @@ def activate_line(x, y, z, command, tile_type=""):
 		g.maps[z].field[y][x].addpix = []
 		return_num = 1
 
+	elif switch == "printvars": #debug: print all variables
+		print g.var_list
+
 	elif switch == "question":  #yes/no dialog box
 		has_dialog = 1
 		if main.show_yesno(command.split(None, 1)[1]):
