@@ -22,6 +22,7 @@ hp = StringVar()
 ep = StringVar()
 
 help_text = StringVar()
+return_from_yesno = StringVar()
 
 #location of the various buttons. (Pixels from the left of the button canvas)
 attack_button_loc = 0
@@ -29,22 +30,33 @@ item_button_loc = 0
 skill_button_loc = 0
 run_button_loc = 0
 
-
-
 #did you run from this battle? Used for scripting.
 did_run = 0
 
 #currently selected button. 0=attack, 1=run/leave, 2=use item, 3=use skill
 cur_button = 0
 
+#Copy of monster.monster_groups[mon_index].monster_list.
+#Used to reduce line length.
+monster_list = []
+
+#Current monster. Used to display stuff like description.
+#Use with monster_list[cur_monster]
+cur_monster = 0
+
+#Number of times you've tried to run from this monster.
+#Used to allow better chances after a while.
+run_attempts = 0
 
 #refreshes the battle view. Call after changing anything.
 def refresh():
-	mon_hp.set(str(g.cur_mon_hp) + "/" + str(monster.monsters[mon_index].hp))
+	mon_hp.set(str(monster_list[cur_monster].hp) + "/" +
+										str(monster_list[cur_monster].maxhp))
 
 	#hp/ep bars
 	canvas_mon_hp.delete("show_mon_hp")
-	temp_width = g.hpbar_width*g.cur_mon_hp/monster.monsters[mon_index].hp
+	temp_width = g.hpbar_width*int(monster_list[cur_monster].hp) / \
+										int(monster_list[cur_monster].maxhp)
 	if temp_width < 0: temp_width=0
 	canvas_mon_hp.create_rectangle(0, 0, temp_width,
 		g.hpbar_width/6, fill="#05BB05", tags="show_mon_hp")
@@ -68,9 +80,9 @@ def refresh():
 #attack button was pressed
 def attack(event=0):
 	global cur_button
-	if g.cur_mon_hp > 0 and g.hp > 0:
+	if monster_list[cur_monster].hp > 0 and g.hp > 0:
 		attack_monster()
-	if g.cur_mon_hp > 0 and g.hp > 0:
+	if monster_list[cur_monster].hp > 0 and g.hp > 0:
 		attack_player()
 		refresh()
 		refresh_buttons()
@@ -80,85 +92,78 @@ def attack_monster():
 	global cur_mon_hp
 	#find the damage done
 	damage = g.die_roll(1, g.adj_attack + 2)
-	damage = damage - g.die_roll(1, monster.monsters[mon_index].defense + 2)
+	damage = damage - g.die_roll(1, monster_list[cur_monster].defense + 2)
 	if damage > 0:
- 		main.print_message("You hit the " + monster.monsters[mon_index].name +
+ 		main.print_message("You hit the " + monster_list[cur_monster].name +
  			" for " + str(damage) + " damage.")
-		g.cur_mon_hp = g.cur_mon_hp - damage
+		monster_list[cur_monster].hp = monster_list[cur_monster].hp - damage
 		#you kill it?
-		if g.cur_mon_hp <= 0:
+		if monster_list[cur_monster].hp <= 0:
 			monster_dead()
 			return 0
 	else:
 		main.print_message("You miss the " +
-				monster.monsters[mon_index].name + ".")
+				monster_list[cur_monster].name + ".")
 
 
 #monster attacks you
 def attack_player():
-	damage = g.die_roll(1, monster.monsters[mon_index].attack + 2)
+	damage = g.die_roll(1, monster_list[cur_monster].attack + 2)
 	damage = damage - g.die_roll(1, g.adj_defense + 2)
 	if damage > 0:
-		main.print_message("The " + monster.monsters[mon_index].name +
+		main.print_message("The " + monster_list[cur_monster].name +
 		" hits you for " + str(damage) + " damage.")
 		g.hp = g.hp - damage
 		#you dead yet?
 		if g.hp <= 0:
-			main.print_message("The " + monster.monsters[mon_index].name +
+			main.print_message("The " + monster_list[cur_monster].name +
 			" kills you.")
 			cur_button = 1
 
-	elif g.cur_mon_hp > 0:
-		main.print_message("The " + monster.monsters[mon_index].name +
+	elif monster_list[cur_monster].hp > 0:
+		main.print_message("The " + monster_list[cur_monster].name +
 		" misses you.")
 	refresh()
 
 #use to reward the player for killing a monster
 def monster_dead():
-	global cur_mon_hp
 	#make the display look better
-	g.cur_mon_hp = 0
+	monster_list[cur_monster].hp = 0
 	#add info to listbox_move; add gold and xp
-	main.print_message("The " + monster.monsters[mon_index].name +
+	main.print_message("The " + monster_list[cur_monster].name +
 	" dies.")
-	gold = monster.monsters[mon_index].gold
+	gold = monster_list[cur_monster].gold
 	g.gold = str(int(g.gold) + gold)
 	main.print_message("You find " + str(gold) + " gold,")
 
-	exp = monster.monsters[mon_index].exp
-	g.exp = int(g.exp) + exp
+	exp = monster_list[cur_monster].exp
 	main.print_message("and get " + str(exp) + " experience.")
+	g.add_exp(exp)
 
-	#gain a level?
-	if int(g.exp) >= int(10*(g.level + 1)*(g.level + 1)):
-		g.level=g.level+1
-		main.print_message("You gain a level.")
-		for i in g.levelup_act:
-			action.activate_line(g.xgrid, g.ygrid, g.zgrid, i)
-
-	#formatting
-	#refresh()
+	#finish up
 	window_battle.destroy()
+	g.cur_window = "main"
 	global did_run
 	did_run = 0
 
 #run/leave/quit button was pressed
 def runaway():
+	global run_attempts
 	try:
-		#if window_battle.state() != "normal":
-		#	return
 		#leave/quit
-		if g.hp <= 0 or g.cur_mon_hp <= 0:
+		if g.hp <= 0 or monster_list[cur_monster].hp <= 0:
 			window_battle.destroy()
-		#run
+		#Attempt to run. Testers get better chance.
 		else:
-			if g.die_roll(1, 10) > 7:
+			if g.die_roll(1, 10+run_attempts) > 7 or \
+					(g.name == "testing123" and g.die_roll(1, 10) > 5):
 				window_battle.destroy()
 				main.print_message("You coward! You ran away from the "
-					+ monster.monsters[mon_index].name + ".")
+					+ monster_list[cur_monster].name + ".")
 				global did_run
 				did_run = 1
 			else:
+				run_attempts += 1
 				main.print_message("You fail to run away.")
 				attack_player()
 	except TclError:
@@ -167,7 +172,7 @@ def runaway():
 #Use item button was pressed
 def useitem():
 	#don't do anything if anyone is dead.
-	if g.cur_mon_hp < 1 or g.hp < 1:
+	if monster_list[cur_monster].hp < 1 or g.hp < 1:
 		return 0
 
 	#put the name of the item into item_name
@@ -197,15 +202,15 @@ def useitem():
 	#if item is explosive
 	if item_type == 12:
 		#if monster is still alive
-		if g.cur_mon_hp > 0:
+		if monster_list[cur_monster].hp > 0:
 			damage = int(g.item.item[item_value].quality)
 			g.item.drop_inv_item(g.item.find_inv_item(item_value))
 			if damage > 0:
-				main.print_message("The " + monster.monsters[mon_index].name +
+				main.print_message("The " + monster_list[cur_monster].name +
 					" is hit for " + str(damage) + " damage.")
-			g.cur_mon_hp = g.cur_mon_hp - damage
+			monster_list[cur_monster].hp = monster_list[cur_monster].hp - damage
 			#you kill it?
-			if g.cur_mon_hp <= 0:
+			if monster_list[cur_monster].hp <= 0:
 				monster_dead()
 				return 0
 			else:
@@ -216,13 +221,15 @@ def useitem():
 	if (listbox_item.size() != 0):
 		listbox_item.selection_set(item_num[0])
 		if (len(listbox_item.curselection()) == 0): #last item
-			listbox_item.selection_set(item_num[0] - 1)
+			listbox_item.selection_set(END)
+			listbox_item.see(END)
+		else: listbox_item.see(item_num[0])
 	refresh_buttons()
 
 
 def useskill():
 	#don't do anything if monster is dead.
-	if g.cur_mon_hp < 1 or g.hp < 1:
+	if monster_list[cur_monster].hp < 1 or g.hp < 1:
 		return 0
 
 	#get the skill name.
@@ -254,31 +261,33 @@ def useskill():
 		attack_player()
 
 	elif g.skill[skill_index][1] == 1: #Sneak away
-		if g.die_roll(1, 10 + int(g.level)) > 4:
+		global run_attempts
+		if g.die_roll(1, 10 + int(g.level) + run_attempts) > 4:
 			main.print_message("You coward! You ran away from the " +
-				monster.monsters[mon_index].name + ".")
+				monster_list[cur_monster].name + ".")
 			window_battle.destroy()
 			global did_run
 			did_run = 1
 			return 0
 		else:
+			run_attempts += 3
 			main.print_message("You fail to sneak away.")
 			attack_player()
 	elif g.skill[skill_index][1] == 2: #Frenzy
 		for i in range(2 + int(g.level)/3):
 			attack_monster()
-			if g.cur_mon_hp <= 0:
-				monster_dead()
+			if monster_list[cur_monster].hp <= 0:
+#				monster_dead()
 				return 0
-		if g.cur_mon_hp > 0:
+		if monster_list[cur_monster].hp > 0:
 			attack_player()
 	elif g.skill[skill_index][1] == 3: #Dismember
 		damage = g.adj_attack * 2
-		main.print_message("You hit the " + monster.monsters[mon_index].name +
+		main.print_message("You hit the " + monster_list[cur_monster].name +
 			" for " + str(damage) + " damage.")
-		g.cur_mon_hp = g.cur_mon_hp - damage
+		monster_list[cur_monster].hp = monster_list[cur_monster].hp - damage
 		#you kill it?
-		if g.cur_mon_hp <= 0:
+		if monster_list[cur_monster].hp <= 0:
 			monster_dead()
 			return 0
 		else:
@@ -291,6 +300,8 @@ def useskill():
 		listbox_skill.selection_set(skill_num[0])
 		if (len(listbox_skill.curselection()) == 0): #last skill
 			listbox_skill.selection_set(END)
+			listbox_skill.see(END)
+		else: listbox_skill.see(skill_num[0])
 	refresh_buttons()
 
 
@@ -301,7 +312,7 @@ def refresh_buttons():
 	if (cur_button == 0):
 		canvas_buttons.create_image(attack_button_loc, 0, anchor=NW,
 					image=attack_button_sel)
-		help_text.set("Attack the " + monster.monsters[mon_index].name)
+		help_text.set("Attack the " + monster_list[cur_monster].name)
 	else: canvas_buttons.create_image(attack_button_loc, 0, anchor=NW,
 					image=attack_button)
 	if (cur_button == 1):
@@ -309,7 +320,7 @@ def refresh_buttons():
 			canvas_buttons.create_image(run_button_loc, 0, anchor=NW,
 				image=quit_button_sel)
 			help_text.set("Quit")
-		elif g.cur_mon_hp > 0:
+		elif monster_list[cur_monster].hp > 0:
 			canvas_buttons.create_image(run_button_loc, 0, anchor=NW,
 				image=run_button_sel)
 			help_text.set("Run away")
@@ -319,7 +330,8 @@ def refresh_buttons():
 			help_text.set("Leave")
 	elif g.hp <= 0: canvas_buttons.create_image(run_button_loc, 0, anchor=NW,
 			image=quit_button)
-	elif g.cur_mon_hp > 0: canvas_buttons.create_image(run_button_loc,
+	elif monster_list[cur_monster].hp > 0:
+		canvas_buttons.create_image(run_button_loc,
 			0, anchor=NW, image=run_button)
 	else: canvas_buttons.create_image(run_button_loc, 0, anchor=NW,
 			image=leave_button)
@@ -509,6 +521,87 @@ def mouse_handler_move(event=0):
 		listbox_skill.select_clear(0, END)
 	refresh_buttons()
 
+
+#Like show_yesno in main, but only expects one or two lines,
+#and creates the canvas used.
+def battle_show_yesno(line=""):
+	action.has_dialog = 1
+	global active_button
+	active_button = 0
+	global return_from_yesno
+	return_from_yesno.set("0")
+
+	canvas_width = no_button.width()*4/3+yes_button.width()*4/3
+	canvas_height = yes_button.height()+50
+	global canvas_battle2
+	canvas_battle2 = Canvas(window_battle, width=canvas_width,
+		height=canvas_height, highlightthickness=0)
+	canvas_battle2.grid(column=1, row=2, columnspan=2, rowspan=7, sticky="S")
+
+	#background
+	canvas_battle2.create_rectangle(0, 0, canvas_width, canvas_height,
+		fill="#5050ff", outline="Black", tags="background_box")
+	#text
+	canvas_battle2.create_text(3, 3, text=line, anchor="nw",
+		width=canvas_width-6, tags="dialogtxt")
+	#buttons
+	canvas_battle2.create_image(canvas_width/2,
+		canvas_height, image=no_button_sel, anchor="se", tags="no")
+	canvas_battle2.create_image(canvas_width/2,
+		canvas_height, image=yes_button, anchor="sw", tags="yes")
+	#bindings
+	main.window_main.bind(g.binding_up, change_yesno_sel)
+	main.window_main.bind(g.binding_right, change_yesno_sel)
+	main.window_main.bind(g.binding_left, change_yesno_sel)
+	main.window_main.bind(g.binding_down, change_yesno_sel)
+	main.window_main.bind(g.binding_action, activate_yesno)
+	canvas_battle2.tag_bind("no", "<Enter>", change_yesno_to_no)
+	canvas_battle2.tag_bind("yes", "<Enter>", change_yesno_to_yes)
+	canvas_battle2.tag_bind("no", "<Button-1>", activate_yesno)
+	canvas_battle2.tag_bind("yes", "<Button-1>", activate_yesno)
+
+	canvas_buttons.bind("<ButtonRelease-1>", do_nothing)
+	canvas_buttons.bind("<Motion>", do_nothing)
+	listbox_item.bind("<ButtonRelease-1>", do_nothing)
+	listbox_skill.bind("<ButtonRelease-1>", do_nothing)
+
+	#wait. Continue after activate_yesno() is run.
+	window_battle.wait_variable(return_from_yesno)
+	#cleanup
+	try: canvas_battle2.destroy()
+	except TclError: return 1
+
+	bind_keys()
+
+	action.has_dialog = 0
+	return active_button
+
+
+#when a yesno dialog box is active, switch to yes.
+def change_yesno_to_yes(event=None):
+	global active_button
+	active_button = 1
+	canvas_battle2.itemconfigure("no", image=no_button)
+	canvas_battle2.itemconfigure("yes", image=yes_button_sel)
+
+#when a yesno dialog box is active, switch to no.
+def change_yesno_to_no(event=None):
+	global active_button
+	active_button = 0
+	canvas_battle2.itemconfigure("no", image=no_button_sel)
+	canvas_battle2.itemconfigure("yes", image=yes_button)
+
+#when a yesno dialog box is active, switch the yes and no buttons.
+def change_yesno_sel(event=None):
+	if active_button == 0: change_yesno_to_yes()
+	else: change_yesno_to_no()
+
+#called on Return when the yesno dialog box is active.
+def activate_yesno(event=None):
+	global return_from_yesno
+	return_from_yesno.set("1")
+
+
 #start a battle.
 def begin(mon_index_input):
 
@@ -519,36 +612,64 @@ def begin(mon_index_input):
 	window_battle.grid(row=1, column=5, columnspan=13, rowspan=13, sticky=E)
 	global mon_index
 	mon_index = mon_index_input
-	global cur_mon_hp
-	g.cur_mon_hp = monster.monsters[mon_index].hp
+	global monster_list
+	monster_list = []
+	for line in monster.monster_groups[mon_index].monster_list:
+		monster_list.append(monster.monsters[monster.monster_name_to_index(line)])
+	for i in range(len(monster_list)):
+		monster_list[i].reset()
+
 	global did_run
 	did_run = 0
 	global cur_button
 	cur_button = 0
+	global cur_monster
+	cur_monster = 0
+	global run_attempts
+	run_attempts = 0
+
+	global yes_button; global yes_button_sel
+	global no_button; global no_button_sel
+	yes_button = ImageTk.PhotoImage(file=g.mod_directory + \
+		"/images/buttons/yes.png")
+	no_button = ImageTk.PhotoImage(file=g.mod_directory + \
+		"/images/buttons/no.png")
+	yes_button_sel = ImageTk.PhotoImage(file=g.mod_directory + \
+                "/images/buttons/yes_sel.png")
+	no_button_sel = ImageTk.PhotoImage(file=g.mod_directory + \
+                "/images/buttons/no_sel.png")
+
+	# Set the current window
+	g.cur_window = "battle"
+
+
 
 	global canvas_desc
 	canvas_desc = Canvas(window_battle, width=200, height=190,
 		highlightthickness=0, bg=bgcolour)
 	canvas_desc.grid(row=2, column=1, sticky=NSEW)
 	canvas_desc.create_text(1, 5, anchor=NW,
-			text=monster.monsters[mon_index].description, width=195)
+			text=monster_list[cur_monster].description, width=195)
 
 	pic_frame = Frame(window_battle, bd=2, relief=SUNKEN, bg="#327a22")
 	pic_frame.grid(row=1, column=2, columnspan=3, rowspan=6, sticky=E)
 
 	global canvas_mon_pic
+	background_pic = ImageTk.PhotoImage(file=g.mod_directory +
+			"/images/monsters/generic.png")
 	if os.path.isfile(g.mod_directory +
-			"/images/monsters/" + monster.monsters[mon_index].name + ".png"):
+			"/images/tiles/monsters/" + monster_list[cur_monster].name + ".png"):
 		monster_pic = ImageTk.PhotoImage(file=g.mod_directory +
-			"/images/monsters/" + monster.monsters[mon_index].name + ".png")
+			"/images/tiles/monsters/" + monster_list[cur_monster].name + ".png")
 	else:
 		monster_pic = ImageTk.PhotoImage(file=g.mod_directory +
-			"/images/monsters/generic.png")
+			"/images/tiles/monsters/generic.png")
 	canvas_mon_pic = Canvas(pic_frame,
-		height=monster_pic.height(),
-		width=monster_pic.width())
+		height=background_pic.height(),
+		width=background_pic.width())
 	canvas_mon_pic.grid(column=2, row=1, rowspan=5, sticky=NW)
-	canvas_mon_pic.create_image(0, 0, anchor=NW, image=monster_pic)
+	canvas_mon_pic.create_image(0, 0, anchor=NW, image=background_pic)
+	canvas_mon_pic.create_image(background_pic.width()/2, 10, anchor=N, image=monster_pic)
 
 
 	global canvas_mon_hp
@@ -561,7 +682,7 @@ def begin(mon_index_input):
 
 	#formatting for the labels.
 	label_formatting = Label(window_battle,
-			text=monster.monsters[mon_index].name, bg=bgcolour).grid(row=1,
+			text=monster_list[cur_monster].name, bg=bgcolour).grid(row=1,
 			column=1, columnspan=2)
 	label_formatting = Label(pic_frame,
 		text="HP:", bg="#327a22").grid(row=5, column=2, sticky=W, padx=2)
@@ -572,10 +693,10 @@ def begin(mon_index_input):
 		bg="#327a22").grid(row=5, column=2, sticky=E, padx=2)
 
 	label_attack = Label(window_battle, text="Attack: " +
-	str(monster.monsters[mon_index].attack), bg=bgcolour).grid(row=4,
+	str(monster_list[cur_monster].attack), bg=bgcolour).grid(row=4,
 			column=1, sticky=W)
 	label_defend = Label(window_battle, text="Defense: " +
-	str(monster.monsters[mon_index].defense), bg=bgcolour).grid(row=4,
+	str(monster_list[cur_monster].defense), bg=bgcolour).grid(row=4,
 			column=1, sticky=E)
 
 	#help text
@@ -672,8 +793,22 @@ def begin(mon_index_input):
 	window_battle.columnconfigure(2, minsize=pic_frame.winfo_width())
 
 
-
 	#bindings
+	bind_keys()
+
+	main.print_message("The " + monster.monster_groups[mon_index].name +
+		" attacks you.")
+
+	refresh()
+	refresh_buttons()
+
+	#keep the window_main window asleep until this one is closed.
+	window_battle.master.wait_window(window_battle)
+	return did_run
+
+
+#bind the keys. Called upon window creation and return from a yes/no box
+def bind_keys():
 	main.window_main.bind(g.binding_attack, attack)
 	main.window_main.bind(g.binding_cancel, key_handler_esc)
 	main.window_main.bind(g.binding_action, key_handler_return)
@@ -685,20 +820,10 @@ def begin(mon_index_input):
 	main.window_main.bind(g.binding_save, key_handler)
 	main.window_main.bind(g.binding_quit, key_handler)
 
-
 	canvas_buttons.bind("<ButtonRelease-1>", mouse_handler_click)
 	canvas_buttons.bind("<Motion>", mouse_handler_move)
 	listbox_item.bind("<ButtonRelease-1>", mouse_handler_item)
 	listbox_skill.bind("<ButtonRelease-1>", mouse_handler_skill)
 
-	main.print_message("The " + monster.monsters[mon_index].name +
-		" attacks you.")
-
-	refresh()
-	refresh_buttons()
-
-	#keep the window_main window asleep until this one is closed.
-	window_battle.master.wait_window(window_battle)
-	return did_run
-
-
+def do_nothing(event=0):
+	pass
