@@ -62,6 +62,7 @@ def inv_savegame():
 def leave_inv():
 	if action.has_dialog == 1: return 0
 	try:
+		g.cur_window = "main"
 		window_inv.destroy()
 	except TclError:
 		pass
@@ -549,6 +550,9 @@ def init_window_inv():
 	global curr_item
 	curr_item = 0
 
+	#Set current window to inventory
+	g.cur_window = "inventory"
+
 	#Inventory canvas
 	temp_canvas_width=(g.tilesize*inv_width)+ ((inv_width+1)*2) + 1
 	temp_canvas_height=(g.tilesize*inv_height)+ ((inv_height+1)*2) + 1
