@@ -102,7 +102,13 @@ def close_window(event=0):
 				window_main.destroy()
 			except TclError:
 				pass
-	else: #if called from inv.
+	elif (g.cur_window == "battle"): #if called from battle window
+		if g.hp < 1 or battle.battle_show_yesno("Leave this game?"):
+			try:
+				window_main.destroy()
+			except TclError:
+				pass
+	elif (g.cur_window == "inventory"): #if called from inventory window
 		if g.hp < 1 or inv.inv_show_yesno("Leave this game?"):
 			try:
 				window_main.destroy()
@@ -110,6 +116,7 @@ def close_window(event=0):
 				pass
 
 
+
 #Activate any scripting associated with a tile. X and Y are absolute coords.
 #Assumes scripting stored in maps[zgrid].field[y][x].actions[]
 def activate_scripting(x, y, z):
@@ -319,10 +326,11 @@ def move_hero(x, y):
 						refresh_tile(g.xgrid+half_mapx, g.ygrid-half_mapy+tilenum)
 
 		else:
-			if y == -1: g.cur_hero = "people/hero_n.png"
-			if y ==  1: g.cur_hero = "people/hero_s.png"
-			if x == -1: g.cur_hero = "people/hero_w.png"
-			if x ==  1: g.cur_hero = "people/hero_e.png"
+			if (tempx == g.xgrid + x) and (tempy == g.ygrid + y):
+				if y == -1: g.cur_hero = "people/hero_n.png"
+				if y ==  1: g.cur_hero = "people/hero_s.png"
+				if x == -1: g.cur_hero = "people/hero_w.png"
+				if x ==  1: g.cur_hero = "people/hero_e.png"
 			refreshhero()
 			recalc_stats()
 		g.allow_move = 1
@@ -373,7 +381,7 @@ def passturn():
 				already_refreshed = 1
 
 
-#Starts a battle. Takes the index of the monster in g.monster.monster
+#Starts a battle. Takes the index of the monster in g.monster.monster_groups
 def start_battle(mon_index):
 	global free_move
 	right.destroy()
@@ -384,7 +392,7 @@ def start_battle(mon_index):
 	else:
 		return_num = 1  #otherwise, you succeeded.
 	#number of free moves after attack:
-	free_move = 3
+	free_move = 4
 	button_inv.config(state=NORMAL)
 	button_save.config(state=NORMAL)
 	create_map()
