@@ -221,7 +221,9 @@ def set_tile(x, y, name):
 				temp_name = name
 		for line_num in range(len(cur_map.field[y][x].onload)):
 			if cur_map.field[y][x].onload[line_num].split()[0].strip().lower() == "walk":
-				cur_map.field[y][x].onload[line_num] = "walk " + walk[temp_name]
+				try: cur_map.field[y][x].onload[line_num] = "walk " + walk[temp_name]
+				#default to walkable.
+				except KeyError: cur_map.field[y][x].onload[line_num] = "walk 1"
 	refresh_map()
 
 #add name item to tile at x, y
