@@ -341,12 +341,12 @@ def init_window():
 	reroll_stats()
 
 	#bindings
-	window_new_game.bind("<Escape>", key_handler_esc)
-	window_new_game.bind("<Return>", key_handler_return)
-	window_new_game.bind("<Up>", key_handler_up)
-	window_new_game.bind("<Down>", key_handler_down)
-	window_new_game.bind("<Left>", key_handler_left)
-	window_new_game.bind("<Right>", key_handler_right)
+	window_new_game.bind(g.binding_cancel, key_handler_esc)
+	window_new_game.bind(g.binding_action, key_handler_return)
+	window_new_game.bind(g.binding_up, key_handler_up)
+	window_new_game.bind(g.binding_down, key_handler_down)
+	window_new_game.bind(g.binding_left, key_handler_left)
+	window_new_game.bind(g.binding_right, key_handler_right)
 	canvas_buttons.bind("<ButtonRelease-1>", mouse_handler_click)
 	canvas_buttons.bind("<Motion>", mouse_handler_move)
 
