@@ -217,10 +217,14 @@ class map:
 			cur_char = 0
 			for character in line:
 				self.field[cur_line].append(tile(character))
-				for line in self.tiles[character][0]:
-					self.field[cur_line][cur_char].onload.append(line)
-				for line in self.tiles[character][1]:
-					self.field[cur_line][cur_char].actions.append(line)
+				try:
+					for line in self.tiles[character][0]:
+						self.field[cur_line][cur_char].onload.append(line)
+					for line in self.tiles[character][1]:
+						self.field[cur_line][cur_char].actions.append(line)
+				except KeyError:
+					print "Tile " + character + " is undefined in map " + \
+							self.name
 				cur_char += 1
 			cur_line += 1
 
